import os


def credential(instance):
    alias = "AZURE_" + instance
    if os.environ.get(alias + "_DRIVER_PATH"):
        driver = os.environ.get(alias + "_DRIVER_PATH")
    else:
        driver = os.environ.get(alias + "_DRIVER")
    connection_kwargs = {
        'database': os.environ[alias + "_DATABASE"],
        'uid': os.environ[alias + "_USERNAME"],
        'server': os.environ[alias + "_HOST"],
        'port': os.environ[alias + "_PORT"],
        'password': os.environ[alias + "_PASSWORD"],
        'driver': driver,
        'TDS_Version': '7.2'
    }
    return connection_kwargs



import pyodbc

import os

import sshtunnel
from sshtunnel import SSHTunnelForwarder

from pyzure.connection.azure_credentials import credential


def execute_query(instance, query, data=None, commit=True, cursor=None, cnxn=None, create_schema=False):
    if not cnxn and not cursor:
        connection_kwargs = credential(instance)

        # Create an SSH tunnel
        ssh_host = os.environ.get("SSH_%s_HOST" % instance)
        ssh_user = os.environ.get("SSH_%s_USER" % instance)
        ssh_path_private_key = os.environ.get("SSH_%s_PATH_PRIVATE_KEY" % instance)
        if ssh_host:
            tunnel = SSHTunnelForwarder(
                (ssh_host, 22),
                ssh_username=ssh_user,
                ssh_private_key=ssh_path_private_key,
                remote_bind_address=(
                    os.environ.get("AZURE_%s_HOST" % instance), int(os.environ.get("AZURE_%s_PORT" % instance))),
                local_bind_address=('localhost', 1433),  # could be any available port
            )
            # Start the tunnel
            try:
                tunnel.start()
                print("Tunnel opened!")
            except sshtunnel.HandlerSSHTunnelForwarderError:
                pass

            connection_kwargs["server"] = "127.0.0.1,1433"
            connection_kwargs["port"] = 1433
        cnxn = pyodbc.connect(**connection_kwargs)

        cursor = cnxn.cursor()

    if data:
        try:
            cursor.execute(query, data)
        except Exception as e:
            if ssh_host:
                tunnel.close()
            print("Error:" + str(e))
            raise
        if commit:
            cnxn.commit()
        result = None
    elif create_schema:
        try:
            cursor.execute(query, data)
        except Exception as e:
            if ssh_host:
                tunnel.close()
            print("Error:" + str(e))
            raise
        cnxn.commit()
        result = None
    else:
        try:
            cursor.execute(query, data)
        except Exception as e:
            if ssh_host:
                tunnel.close()
            print("Error:" + str(e))
            raise
        result = []
        try:
            columns = [column[0] for column in cursor.description]
            for row in cursor.fetchall():
                dict_ = dict()
                for i in range(len(columns)):
                    dict_[columns[i]] = row[i]
                result.append(dict_)
        except pyodbc.ProgrammingError:
            pass
        except TypeError:
            cnxn.commit()
            return []
        cnxn.commit()

    if commit:
        cursor.close()
        cnxn.close()
        if ssh_host:
            tunnel.close()
            print("Tunnel closed!")
        return result
    else:
        return result, cursor, cnxn



import os

import time

import pyzure
from pyzure.connection.connection import connect
from pyzure.tools.read_and_write import read_file
from pyzure.tools.print_colors import C


def cleaning_function(instance, table_name, dev=False):
    print("Get info on table...")
    columns = get_table_info(instance, table_name)
    print("Get info on table...OK")
    cnxn = connect(instance)
    cursor = cnxn.cursor()
    drop_request = '''DROP TABLE ''' + table_name + ''';'''
    print(C.WARNING + "Drop table " + C.ENDC)
    cursor.execute(drop_request)
    cnxn.commit()
    cursor.close()
    cnxn.close()
    print("Create table from info...")
    create_table_from_info(instance, columns, table_name)
    print("Create table from info...OK")
    print(C.OKBLUE + "Cleaning Done" + C.ENDC)
    return 0


def get_table_info(instance, table_and_schema_name):
    split = table_and_schema_name.split(".")
    if len(split) == 1:
        table_name = split[0]
        schema_name = None

    elif len(split) == 2:
        table_name = split[1]
        schema_name = split[0]
    else:
        raise Exception("Invalid table or schema name")
    query = "SELECT column_name, data_type, character_maximum_length, is_nullable FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME='%s'" % table_name
    if schema_name:
        query = query + " AND TABLE_SCHEMA='%s'" % schema_name
    print(query)
    return pyzure.execute_query(instance, query)


def create_table_from_info(instance, columns, table_name):
    columns_name_string = []
    for c in columns:
        # COLUMN NAME
        c_string = c["column_name"]

        # DATA TYPE TREATMENT
        data_type = c["data_type"]
        if data_type in ("varchar"):
            data_type = data_type + "(" + str(c["character_maximum_length"]) + ")"

        c_string = c_string + " " + data_type

        # NULLABLE ?
        if c["is_nullable"] == "NO":
            c_string = c_string + " NOT NULL"

        columns_name_string.append(c_string)

    query = "CREATE TABLE %s (%s)" % (table_name, ", ".join(columns_name_string))
    print(query)
    pyzure.execute_query(instance, query)


def commit_function(cnxn):
    cnxn.commit()
    # print(C.OKGREEN + "Committed" + C.ENDC)


def print_progress_bar(iteration, total, prefix='', suffix='', decimals=1, length=100, fill='█'):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    # filled_length = int(length * iteration // total)
    # bar = fill * filled_length + '-' * (length - filled_length)
    # print('\r %s |%s| %s %s' % (prefix, bar, percent, suffix), end='\r')
    print(percent)
    # Print New Line on Complete
    if iteration == total:
        print()


def print_progress_bar_multi_threads(nb_threads, suffix='', decimals=1, length=15,
                                     fill='█'):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    string = ""
    for k in range(nb_threads):
        try:
            threads_state = eval(read_file("threads_state_%s" % str(k)))
        except SyntaxError:
            time.sleep(0.001)
            try:
                threads_state = eval(read_file("threads_state_%s" % str(k)))
            except SyntaxError:
                pass

        iteration = threads_state["iteration"]
        total = threads_state["total"]
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        # filled_length = int(length * iteration // total)
        # bar = fill * filled_length + '-' * (length - filled_length)
        prefix = "Thread %s :" % str(k)
        string = string + '%s %s%% ' % (prefix, percent)

    print(string + " " + suffix)
    # # Print New Line on Complete
    # if iteration == total:
    #     print()
